import _defineProperty from '@babel/runtime/helpers/defineProperty';
import _regeneratorRuntime from '@babel/runtime/regenerator';
import _slicedToArray from '@babel/runtime/helpers/slicedToArray';
import { useEffect, memo, createElement, forwardRef, useReducer, useRef, useMemo, useCallback, Fragment } from 'react';
import _typeof from '@babel/runtime/helpers/typeof';

var useHotkeys = function useHotkeys(_ref) {
  var active = _ref.active,
      handlers = _ref.handlers;
  useEffect(function () {
    var handleKeyDown = function handleKeyDown(event) {
      if (event.code in handlers) {
        handlers[event.code](event);
      }
    };

    if (active) {
      document.addEventListener('keydown', handleKeyDown);
    }

    return function () {
      document.removeEventListener('keydown', handleKeyDown);
    };
  }, [active, handlers]);
};

var NOOP = function NOOP() {
  return void 0;
};

var useOutsideClick = function useOutsideClick(_ref) {
  var containerRef = _ref.containerRef,
      _ref$onInsideClick = _ref.onInsideClick,
      onInsideClick = _ref$onInsideClick === void 0 ? NOOP : _ref$onInsideClick,
      _ref$onOutsideClick = _ref.onOutsideClick,
      onOutsideClick = _ref$onOutsideClick === void 0 ? NOOP : _ref$onOutsideClick;
  useEffect(function () {
    var handleDocumentClick = function handleDocumentClick(event) {
      if (containerRef && containerRef.current) {
        var isOutsideClick = !containerRef.current.contains(event.target);
        isOutsideClick ? onOutsideClick(event) : onInsideClick(event);
      }
    };

    document.addEventListener('mousedown', handleDocumentClick, true);
    return function () {
      document.removeEventListener('mousedown', handleDocumentClick, true);
    };
  }, [containerRef, onInsideClick, onOutsideClick]);
};

var classNames = function classNames() {
  var classes = [];
  var len = arguments.length;

  for (var i = 0; i < len; i += 1) {
    var arg = i < 0 || arguments.length <= i ? undefined : arguments[i];

    if (typeof arg === 'string' && arg.length) {
      // string args will be applied directly, assuming they are not empty
      classes.push(arg);
    } else if (_typeof(arg) === 'object') {
      // object args will be treated as conditional classes
      Object.entries(arg).forEach(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            className = _ref2[0],
            condition = _ref2[1];

        if (condition) {
          classes.push(className);
        }
      });
    }
  }

  return classes.join(' ');
};

// eslint-disable-next-line @typescript-eslint/no-explicit-any
var debounce = function debounce(fn, timeout) {
  var debounceTimeout = Math.max(timeout, 1);
  var currentTimeout;
  return function () {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (currentTimeout) {
      clearTimeout(currentTimeout);
    }

    return new Promise(function (resolve) {
      currentTimeout = setTimeout(function () {
        resolve(fn.apply(void 0, args));
      }, debounceTimeout);
    });
  };
};

/* eslint-disable sort-keys */
var Keys = {
  A: 'KeyA',
  B: 'KeyB',
  C: 'KeyC',
  D: 'KeyD',
  E: 'KeyE',
  F: 'KeyF',
  G: 'KeyG',
  H: 'KeyH',
  I: 'KeyI',
  J: 'KeyJ',
  K: 'KeyK',
  L: 'KeyL',
  M: 'KeyM',
  N: 'KeyN',
  O: 'KeyO',
  P: 'KeyP',
  Q: 'KeyQ',
  R: 'KeyR',
  S: 'KeyS',
  T: 'KeyT',
  U: 'KeyU',
  V: 'KeyV',
  W: 'KeyW',
  X: 'KeyX',
  Y: 'KeyY',
  Z: 'KeyZ',
  '0': 'Digit0',
  '1': 'Digit1',
  '2': 'Digit2',
  '3': 'Digit3',
  '4': 'Digit4',
  '5': 'Digit5',
  '6': 'Digit6',
  '7': 'Digit7',
  '8': 'Digit8',
  '9': 'Digit9',
  F1: 'F1',
  F2: 'F2',
  F3: 'F3',
  F4: 'F4',
  F5: 'F5',
  F6: 'F6',
  F7: 'F7',
  F8: 'F8',
  F9: 'F9',
  F0: 'F0',
  ArrowLeft: 'ArrowLeft',
  ArrowRight: 'ArrowRight',
  ArrowUp: 'ArrowUp',
  ArrowDown: 'ArrowDown',
  Esc: 'Escape',
  Tab: 'Tab',
  Space: 'Space',
  Enter: 'Enter',
  Backspace: 'Backspace',
  CapsLock: 'CapsLock',
  ContextMenu: 'ContextMenu',
  ShiftLeft: 'ShiftLeft',
  ShiftRight: 'ShiftRight',
  ControlLeft: 'ControlLeft',
  ControlRight: 'ControlRight',
  AltLeft: 'AltLeft',
  AltRight: 'AltRight',
  MetaLeft: 'MetaLeft',
  MeatRight: 'MetaRight',
  Num1: 'Numpad1',
  Num2: 'Numpad2',
  Num3: 'Numpad3',
  Num4: 'Numpad4',
  Num5: 'Numpad5',
  Num6: 'Numpad6',
  Num7: 'Numpad7',
  Num8: 'Numpad8',
  Num9: 'Numpad9',
  Num0: 'Numpad0',
  NumLock: 'NumLock',
  NumpadDivide: 'NumpadDivide',
  NumpadMultiply: 'NumpadMultiply',
  NumpadSubtract: 'NumpadSubtract',
  NumpadAdd: 'NumpadAdd',
  NumpadEnter: 'NumpadEnter',
  NumpadDecimal: 'NumpadDecimal',
  Backquote: 'Backquote',
  Backslash: 'Backslash',
  BracketLeft: 'BracketLeft',
  BracketRight: 'BracketRight',
  Comma: 'Comma',
  Delete: 'Delete',
  End: 'End',
  Equal: 'Equal',
  Home: 'Home',
  Insert: 'Insert',
  Minus: 'Minus',
  PageDown: 'PageDown',
  PageUp: 'PageUp',
  Period: 'Period',
  Quote: 'Quote',
  Semicolon: 'Semicolon',
  Slash: 'Slash'
};

var wrap = function wrap(min, max, val) {
  if (val < min) return max;
  if (val > max) return min;
  return val;
};

/* eslint-disable @typescript-eslint/no-explicit-any */
var DEFAULT_CSS_CLASS_BASE = 'reactSimpleSelect';
var DEFAULT_NO_OPTIONS_MESSAGE = 'No Options';
var DEFAULT_PLACEHOLDER = 'Select...';
var DEFAULT_ASYNC_SEARCH_DEBOUNCE = 500;
var DEFAULT_ASYNC_SEARCHING_TEXT = 'Searching...';
var DEFAULT_GET_OPTION_KEY = function DEFAULT_GET_OPTION_KEY(option) {
  return option;
};
var DEFAULT_GET_OPTION_VALUE = function DEFAULT_GET_OPTION_VALUE(option) {
  return option;
};
var DEFAULT_GET_OPTION_LABEL = function DEFAULT_GET_OPTION_LABEL(option) {
  return "".concat(option);
};
var DEFAULT_TEST_ID_CLEAR_ICON = "".concat(DEFAULT_CSS_CLASS_BASE, "-icon-clear");
/**
 * A helper function to generate the full CSS class name
 * on top of the shared base class name.
 *
 * e.g. If base class name is `reactSimpleSelect`, calling this with `__label`
 * would return teh full string `reactSimpleSelect__label`.
 * @param className
 */

var css = function css(className) {
  return "".concat(DEFAULT_CSS_CLASS_BASE).concat(className);
};
/**
 * A map of pre-built style names for the default styling for the Select component.
 * This is mostly a convenience to simplify and shorten the application of some
 * fairly long and repetitive names. It also gives a "faux CSS Modules" style
 * naming convention, of being able to apply CSS classes with type completion.
 *
 * In general, these should be used when applying default styles to Select,
 * rather than setting them as strings.
 *
 * Something like:
 * `<div className={styles.active} />`
 */

var styles = {
  active: css('__active'),
  container: css('__container'),
  currentValue: css('__currentValue'),
  disabled: css('__disabled'),
  hidden: css('__hidden'),
  highlighted: css('__highlighted'),
  inputWrapper: css('__inputWrapper'),
  label: css('__label'),
  labelWrapper: css('__labelWrapper'),
  loader: css('__loader'),
  loaderItem: css('__loaderItem'),
  loaderWrapper: css('__loaderWrapper'),
  noOptionsMessage: css('__noOptionsMessage'),
  option: css('__option'),
  optionsWrapper: css('__optionsWrapper'),
  placeholder: css('__placeholder'),
  selectInput: css('__selectInput'),
  selected: css('__selected'),
  svgWrapper: css('__svgWrapper')
};
/**
 * A simple default search/filter function for the provided set of options.
 * Performs a simple case-insensitive search between the search term
 * and the parsed label of each option in the current set.
 * @param props
 * @param searchString
 */

var filterOptionsBySearch = function filterOptionsBySearch(props, searchString) {
  var _props$getOptionLabel;

  var _props$options = props.options,
      options = _props$options === void 0 ? [] : _props$options;
  var getOptionLabel = (_props$getOptionLabel = props.getOptionLabel) !== null && _props$getOptionLabel !== void 0 ? _props$getOptionLabel : DEFAULT_GET_OPTION_LABEL;

  var filterOptions = function filterOptions(option) {
    return getOptionLabel(option).toLowerCase().includes(searchString.toLowerCase());
  };

  return options.filter(filterOptions);
};

var Loader = memo(function (_ref) {
  var _ref$testIds = _ref.testIds,
      testIds = _ref$testIds === void 0 ? {} : _ref$testIds;
  return createElement("div", {
    className: styles.loaderWrapper,
    "data-testid": testIds.loaderWrapper
  }, createElement("div", {
    className: styles.loader,
    "data-testid": testIds.loader
  }, createElement("div", {
    className: styles.loaderItem
  }), createElement("div", {
    className: styles.loaderItem
  }), createElement("div", {
    className: styles.loaderItem
  }), createElement("div", {
    className: styles.loaderItem
  })));
});

var SvgWrapper = forwardRef(function (_ref, ref) {
  var children = _ref.children,
      onMouseDown = _ref.onMouseDown,
      testId = _ref.testId;
  return createElement("div", {
    className: classNames(styles.svgWrapper),
    "data-testid": testId,
    onMouseDown: onMouseDown,
    ref: ref
  }, children);
});

var ClearX = memo(function () {
  return createElement("svg", {
    "aria-hidden": "true",
    focusable: "false",
    height: "20",
    viewBox: "0 0 20 20",
    width: "20"
  }, createElement("path", {
    d: "M14.348 14.849c-0.469 0.469-1.229 0.469-1.697 0l-2.651-3.030-2.651 3.029c-0.469 0.469-1.229 0.469-1.697 0-0.469-0.469-0.469-1.229 0-1.697l2.758-3.15-2.759-3.152c-0.469-0.469-0.469-1.228 0-1.697s1.228-0.469 1.697 0l2.652 3.031 2.651-3.031c0.469-0.469 1.228-0.469 1.697 0s0.469 1.229 0 1.697l-2.758 3.152 2.758 3.15c0.469 0.469 0.469 1.229 0 1.698z"
  }));
});

var DownArrowSVG = memo(function () {
  return createElement("svg", {
    "aria-hidden": "true",
    focusable: "false",
    height: "20",
    viewBox: "0 0 20 20",
    width: "20"
  }, createElement("path", {
    d: "M4.516 7.548c0.436-0.446 1.043-0.481 1.576 0l3.908 3.747 3.908-3.747c0.533-0.481 1.141-0.446 1.574 0 0.436 0.445 0.408 1.197 0 1.615-0.406 0.418-4.695 4.502-4.695 4.502-0.217 0.223-0.502 0.335-0.787 0.335s-0.57-0.112-0.789-0.335c0 0-4.287-4.084-4.695-4.502s-0.436-1.17 0-1.615z"
  }));
});

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }
var SelectActionType;

(function (SelectActionType) {
  SelectActionType[SelectActionType["focus"] = 0] = "focus";
  SelectActionType[SelectActionType["blur"] = 1] = "blur";
  SelectActionType[SelectActionType["inputChange"] = 2] = "inputChange";
  SelectActionType[SelectActionType["openMenu"] = 3] = "openMenu";
  SelectActionType[SelectActionType["closeMenu"] = 4] = "closeMenu";
  SelectActionType[SelectActionType["setHighlighted"] = 5] = "setHighlighted";
  SelectActionType[SelectActionType["asyncSearchStart"] = 6] = "asyncSearchStart";
  SelectActionType[SelectActionType["asyncSearchEnd"] = 7] = "asyncSearchEnd";
})(SelectActionType || (SelectActionType = {}));

var initialSelectState = function initialSelectState() {
  var options = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
  return {
    active: false,
    highlightedIdx: 0,
    inputValue: '',
    menuIsOpen: false,
    searching: false,
    visibleOptions: options
  };
};
var reducer = function reducer(state, action) {
  switch (action.type) {
    case SelectActionType.focus:
      return _objectSpread({}, state, {
        active: true
      });

    case SelectActionType.blur:
      {
        var visibleOptions = action.props.options || [];
        return _objectSpread({}, state, {
          active: false,
          highlightedIdx: 0,
          inputValue: '',
          menuIsOpen: false,
          visibleOptions: visibleOptions
        });
      }

    case SelectActionType.inputChange:
      {
        var _ref, _action$payload;

        var props = action.props || {}; // eslint-disable-line

        var value = (_ref = (_action$payload = action.payload) === null || _action$payload === void 0 ? void 0 : _action$payload.inputValue) !== null && _ref !== void 0 ? _ref : '';

        var _visibleOptions = filterOptionsBySearch(props, value);

        return _objectSpread({}, state, {
          highlightedIdx: 0,
          inputValue: value,
          menuIsOpen: true,
          searching: false,
          visibleOptions: _visibleOptions
        });
      }

    case SelectActionType.openMenu:
      {
        return _objectSpread({}, state, {
          active: true,
          highlightedIdx: 0,
          menuIsOpen: true
        });
      }

    case SelectActionType.closeMenu:
      {
        var _visibleOptions2 = action.props.options || [];

        return _objectSpread({}, state, {
          highlightedIdx: 0,
          inputValue: '',
          menuIsOpen: false,
          visibleOptions: _visibleOptions2
        });
      }

    case SelectActionType.setHighlighted:
      {
        var _action$payload2, _action$payload3;

        var newIdx = ((_action$payload2 = action.payload) === null || _action$payload2 === void 0 ? void 0 : _action$payload2.highlightIdx) || state.highlightedIdx + (((_action$payload3 = action.payload) === null || _action$payload3 === void 0 ? void 0 : _action$payload3.highlightIncrement) || 0);
        var highlightedIdx = wrap(0, state.visibleOptions.length - 1, newIdx);

        if (highlightedIdx === state.highlightedIdx) {
          return state;
        }

        return _objectSpread({}, state, {
          highlightedIdx: highlightedIdx
        });
      }

    case SelectActionType.asyncSearchStart:
      {
        var _ref2, _action$payload4;

        var newInputValue = (_ref2 = (_action$payload4 = action.payload) === null || _action$payload4 === void 0 ? void 0 : _action$payload4.inputValue) !== null && _ref2 !== void 0 ? _ref2 : '';
        return _objectSpread({}, state, {
          inputValue: newInputValue,
          menuIsOpen: true,
          searching: true,
          visibleOptions: []
        });
      }

    case SelectActionType.asyncSearchEnd:
      {
        var _action$payload5;

        if (!state.searching) return state;
        var newOptions = ((_action$payload5 = action.payload) === null || _action$payload5 === void 0 ? void 0 : _action$payload5.options) || [];
        return _objectSpread({}, state, {
          searching: false,
          visibleOptions: newOptions
        });
      }

    default:
      return state;
  }
};

var nextId = 0;
var Select = memo(function (props) {
  var _handlers, _classNames2, _classNames3, _testIds$clearSelecti;

  var asyncSearch = props.asyncSearch,
      _props$asyncSearchDeb = props.asyncSearchDebounceTime,
      asyncSearchDebounceTime = _props$asyncSearchDeb === void 0 ? DEFAULT_ASYNC_SEARCH_DEBOUNCE : _props$asyncSearchDeb,
      _props$asyncSearchMin = props.asyncSearchMinLength,
      asyncSearchMinLength = _props$asyncSearchMin === void 0 ? 1 : _props$asyncSearchMin,
      _props$asyncSearching = props.asyncSearchingText,
      asyncSearchingText = _props$asyncSearching === void 0 ? DEFAULT_ASYNC_SEARCHING_TEXT : _props$asyncSearching,
      _props$clearable = props.clearable,
      clearable = _props$clearable === void 0 ? true : _props$clearable,
      _props$disabled = props.disabled,
      disabled = _props$disabled === void 0 ? false : _props$disabled,
      _props$getOptionKey = props.getOptionKey,
      getOptionKey = _props$getOptionKey === void 0 ? DEFAULT_GET_OPTION_KEY : _props$getOptionKey,
      _props$getOptionLabel = props.getOptionLabel,
      getOptionLabel = _props$getOptionLabel === void 0 ? DEFAULT_GET_OPTION_LABEL : _props$getOptionLabel,
      getOptionTestId = props.getOptionTestId,
      _props$getOptionValue = props.getOptionValue,
      getOptionValue = _props$getOptionValue === void 0 ? DEFAULT_GET_OPTION_VALUE : _props$getOptionValue,
      label = props.label,
      _props$noOptionsMessa = props.noOptionsMessage,
      noOptionsMessage = _props$noOptionsMessa === void 0 ? DEFAULT_NO_OPTIONS_MESSAGE : _props$noOptionsMessa,
      onChange = props.onChange,
      _props$options = props.options,
      options = _props$options === void 0 ? [] : _props$options,
      optionIsDisabled = props.optionIsDisabled,
      _props$placeholder = props.placeholder,
      placeholder = _props$placeholder === void 0 ? DEFAULT_PLACEHOLDER : _props$placeholder,
      _props$searchable = props.searchable,
      searchable = _props$searchable === void 0 ? true : _props$searchable,
      _props$testIds = props.testIds,
      testIds = _props$testIds === void 0 ? {} : _props$testIds,
      value = props.value;

  var _React$useReducer = useReducer(reducer, initialSelectState(options)),
      _React$useReducer2 = _slicedToArray(_React$useReducer, 2),
      state = _React$useReducer2[0],
      dispatch = _React$useReducer2[1];

  var active = state.active,
      highlightedIdx = state.highlightedIdx,
      inputValue = state.inputValue,
      menuIsOpen = state.menuIsOpen,
      searching = state.searching,
      visibleOptions = state.visibleOptions; // eslint-disable-next-line no-plusplus

  var id = useRef(++nextId); // unique ID for each instance

  var labelId = useMemo(function () {
    return "reactSimpleSelect__label--".concat(id.current);
  }, [id]);
  var containerRef = useRef(null); // ref to the main wrapper element

  var inputRef = useRef(null); // ref to the <input> element

  var menuRef = useRef(null); // ref to the options menu/list

  var closeIconRef = useRef(null); // refs that should prevent the handling of "inside click" callbacks when the menu is open
  // menuRef: this needs to process "click" on its own, and "mouseDown" will cause the menu to disappear too early

  var openMenuMouseDownStopRefs = [menuRef]; // refs that should prevent the handling of "inside click" callbacks when the menu is closed
  // closeIconRef: this should clear selection, but not open the menu

  var closedMenuMouseDownStopRefs = [closeIconRef];
  /** Text value to display in the "current value" field; either the current value or placeholder text */

  var displayValue = value && getOptionLabel(value) || placeholder;
  var shouldRenderClearIcon = !disabled && clearable && value;

  var focusInputElement = function focusInputElement() {
    if (inputRef === null || inputRef === void 0 ? void 0 : inputRef.current) {
      inputRef.current.focus();
    }
  };

  var blurInputElement = function blurInputElement() {
    if (inputRef === null || inputRef === void 0 ? void 0 : inputRef.current) {
      inputRef.current.blur();
    }
  };

  var openMenu = useCallback(function () {
    dispatch({
      props: props,
      type: SelectActionType.openMenu
    });
  }, [props]);
  var closeMenu = useCallback(function () {
    dispatch({
      props: props,
      type: SelectActionType.closeMenu
    });
    setTimeout(focusInputElement, 0);
  }, [props]);
  var blur = useCallback(function () {
    dispatch({
      props: props,
      type: SelectActionType.blur
    });
    blurInputElement();
  }, [props]);
  /**
   * Debounced async search function
   * Will attempt to trigger the async search prop when specified.
   * Any errors caught here will be gracefully ignored, based on the assumption
   * that the error is being properly handled in the parent component.
   */

  var debouncedAsyncSearch = asyncSearch ? useCallback(debounce(function _callee(searchString) {
    var newOptions;
    return _regeneratorRuntime.async(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            newOptions = [];
            _context.prev = 1;
            _context.next = 4;
            return _regeneratorRuntime.awrap(asyncSearch(searchString));

          case 4:
            newOptions = _context.sent;
            _context.next = 9;
            break;

          case 7:
            _context.prev = 7;
            _context.t0 = _context["catch"](1);

          case 9:
            _context.prev = 9;
            dispatch({
              payload: {
                options: newOptions
              },
              props: props,
              type: SelectActionType.asyncSearchEnd
            });
            return _context.finish(9);

          case 12:
          case "end":
            return _context.stop();
        }
      }
    }, null, null, [[1, 7, 9, 12]]);
  }, asyncSearchDebounceTime), [asyncSearch, props]) : function () {
    return void 0;
  }; // ============================================================
  //  Highlight/selection management
  // ============================================================

  var decrementHighlightedOption = useCallback(function () {
    dispatch({
      payload: {
        highlightIncrement: -1
      },
      props: props,
      type: SelectActionType.setHighlighted
    });
  }, [props]);
  var incrementHighlightedOption = useCallback(function () {
    dispatch({
      payload: {
        highlightIncrement: 1
      },
      props: props,
      type: SelectActionType.setHighlighted
    });
  }, [props]);

  var setHighlightedOption = function setHighlightedOption(highlightIdx) {
    dispatch({
      payload: {
        highlightIdx: highlightIdx
      },
      props: props,
      type: SelectActionType.setHighlighted
    });
  };

  var setSelectedOption = useCallback(function (option) {
    onChange(option);
    closeMenu();
  }, [closeMenu, onChange]);
  /**
   * Attempts to call the reducer to update the currently-highlighted option.
   * If the currently-highlighted option is disabled, nothing will happen.
   */

  var selectHighlightedOption = useCallback(function () {
    var option = visibleOptions[highlightedIdx];

    if (option) {
      if (optionIsDisabled && optionIsDisabled(option, highlightedIdx)) {
        return;
      }

      setSelectedOption(option);
    }
  }, [highlightedIdx, optionIsDisabled, setSelectedOption, visibleOptions]); // ============================================================
  //  Keyboard handlers
  // ============================================================

  var handleUpKey = useCallback(function () {
    if (menuIsOpen) {
      decrementHighlightedOption();
    } else {
      openMenu();
    }
  }, [decrementHighlightedOption, menuIsOpen, openMenu]);
  var handleDownKey = useCallback(function () {
    if (menuIsOpen) {
      incrementHighlightedOption();
    } else {
      openMenu();
    }
  }, [incrementHighlightedOption, menuIsOpen, openMenu]);
  var handleEnterKey = useCallback(function () {
    selectHighlightedOption();
  }, [selectHighlightedOption]);
  var handleEscKey = useCallback(function () {
    closeMenu();
  }, [closeMenu]);
  var handleTabKey = useCallback(function (event) {
    if (menuIsOpen && visibleOptions.length) {
      event.preventDefault();
      event.stopPropagation();
      selectHighlightedOption();
    } else {
      blur();
    }
  }, [blur, menuIsOpen, selectHighlightedOption, visibleOptions]);

  var handleInputChange = function handleInputChange(event) {
    var _ref, _ref$value, inputValue;

    return _regeneratorRuntime.async(function handleInputChange$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _ref = event === null || event === void 0 ? void 0 : event.target, _ref$value = _ref.value, inputValue = _ref$value === void 0 ? '' : _ref$value;

            if (asyncSearch && inputValue.length >= asyncSearchMinLength) {
              dispatch({
                payload: {
                  inputValue: inputValue
                },
                props: props,
                type: SelectActionType.asyncSearchStart
              });
              debouncedAsyncSearch(inputValue);
            } else {
              dispatch({
                payload: {
                  inputValue: inputValue
                },
                props: props,
                type: SelectActionType.inputChange
              });
            }

          case 2:
          case "end":
            return _context2.stop();
        }
      }
    });
  }; // callback for handling clicks within the parent element
  // note that this callback is triggered by the `useOutsideClick` hook


  var handleInsideClick = function handleInsideClick(event) {
    if (disabled) return;
    var refs = menuIsOpen ? openMenuMouseDownStopRefs : closedMenuMouseDownStopRefs;

    for (var i = 0; i < refs.length; i += 1) {
      var ref = refs[i].current;

      if (ref && ref.contains(event.target)) {
        return;
      }
    }

    if (menuIsOpen) {
      closeMenu();
    } else {
      openMenu();
    }
  };

  var handOutsideClick = function handOutsideClick(_event) {
    blur();
  };

  var handleInputFocus = function handleInputFocus() {
    if (disabled || active) return;
    dispatch({
      props: props,
      type: SelectActionType.focus
    });
  };
  /**
   * A handler simply to prevent the initial 'mousedown' event from stealing focus
   * from the <input>, thus preventing an unwanted blur.
   *
   * The selecting of the option will handle in the 'click' handler, but the browser
   * will fire this AFTER the <input> element blurs, so we need to manually prevent
   * that from happening.
   * @param event
   */


  var handleOptionMouseDown = function handleOptionMouseDown(event) {
    event.preventDefault();
    event.stopPropagation();
  };

  var handleOptionMouseOver = function handleOptionMouseOver(event, idx) {
    setHighlightedOption(idx);
  };
  /**
   * Handler for clicking on an option in the menu. Unlike the 'mousedown' handler,
   * this one actually does fire the 'onChange' callback with the selected option,
   * and then triggers the 'blur' action itself.
   */


  var handleOptionClick = useCallback(function (option) {
    onChange(option);
    closeMenu();
  }, [closeMenu, onChange]);
  var handleClearClick = useCallback(function () {
    onChange(undefined);
    closeMenu();
  }, [closeMenu, onChange]);
  /**
   * Effect handler to focus the <input> element any time the wrapper is activated.
   */

  useEffect(function () {
    if (active) {
      setTimeout(focusInputElement, 0);
    }
  }, [active]);
  useOutsideClick({
    containerRef: containerRef,
    onInsideClick: handleInsideClick,
    onOutsideClick: handOutsideClick
  });
  useHotkeys({
    active: active,
    handlers: (_handlers = {}, _defineProperty(_handlers, Keys.ArrowUp, handleUpKey), _defineProperty(_handlers, Keys.ArrowDown, handleDownKey), _defineProperty(_handlers, Keys.Enter, handleEnterKey), _defineProperty(_handlers, Keys.Esc, handleEscKey), _defineProperty(_handlers, Keys.Tab, handleTabKey), _handlers)
  });
  return createElement(Fragment, null, label && createElement("div", {
    className: classNames(styles.labelWrapper),
    "data-testid": testIds.labelWrapper
  }, createElement("label", {
    className: classNames(styles.label),
    "data-testid": testIds.label,
    id: labelId
  }, label)), createElement("div", {
    "aria-labelledby": label ? labelId : undefined,
    className: classNames(styles.container, _defineProperty({}, styles.disabled, disabled)),
    "data-testid": testIds.container,
    ref: containerRef
  }, createElement("div", {
    className: classNames(styles.inputWrapper, (_classNames2 = {}, _defineProperty(_classNames2, styles.active, active), _defineProperty(_classNames2, styles.disabled, disabled), _classNames2)),
    "data-testid": testIds.inputWrapper
  }, createElement("div", {
    className: classNames(styles.currentValue, (_classNames3 = {}, _defineProperty(_classNames3, styles.hidden, !!inputValue), _defineProperty(_classNames3, styles.placeholder, !value), _classNames3)),
    "data-testid": testIds.currentValue
  }, displayValue), createElement("input", {
    className: classNames(styles.selectInput, _defineProperty({}, styles.disabled, disabled)),
    "data-testid": testIds.selectInput,
    disabled: disabled,
    onChange: handleInputChange,
    onFocus: handleInputFocus,
    readOnly: !searchable,
    ref: inputRef,
    type: "text",
    value: inputValue
  }), searching && createElement(Loader, {
    testIds: testIds
  }), shouldRenderClearIcon && createElement(SvgWrapper, {
    onMouseDown: handleClearClick,
    ref: closeIconRef,
    testId: (_testIds$clearSelecti = testIds.clearSelection) !== null && _testIds$clearSelecti !== void 0 ? _testIds$clearSelecti : DEFAULT_TEST_ID_CLEAR_ICON
  }, createElement(ClearX, null)), createElement(SvgWrapper, null, createElement(DownArrowSVG, null))), menuIsOpen && createElement("div", {
    className: classNames(styles.optionsWrapper),
    "data-testid": testIds.optionsWrapper,
    ref: menuRef
  }, visibleOptions.length ? visibleOptions.map(function (option, idx) {
    var _classNames5;

    var key = getOptionKey ? getOptionKey(option) : idx;
    var testId = getOptionTestId && getOptionTestId(option, idx);
    var isHighlighted = idx === highlightedIdx;
    var isSelected = getOptionValue(option) === getOptionValue(value);
    var isDisabled = optionIsDisabled ? optionIsDisabled(option, idx) : false;
    return createElement("div", {
      className: classNames(styles.option, (_classNames5 = {}, _defineProperty(_classNames5, styles.highlighted, isHighlighted), _defineProperty(_classNames5, styles.selected, isSelected), _defineProperty(_classNames5, styles.disabled, isDisabled), _classNames5)),
      "data-testid": testId,
      key: key,
      onClick: isDisabled ? undefined : function () {
        return handleOptionClick(option);
      },
      onMouseDown: isDisabled ? undefined : handleOptionMouseDown,
      onMouseOver: isDisabled ? undefined : function (event) {
        return handleOptionMouseOver(event, idx);
      }
    }, getOptionLabel(option));
  }) : createElement("div", {
    className: classNames(styles.option, styles.noOptionsMessage),
    "data-testid": testIds.noOptionsMessage
  }, searching ? asyncSearchingText : noOptionsMessage))));
});

export { Select };
//# sourceMappingURL=react-simple-select.esm.js.map
